function edf_crop(fnametemplate,fnamerange,xrange,yrange,varargin)
% EDF_CROP.M
% Crops sequences of EDF files (orignalXYZ.edf) and writes them to 'originalcrop_XYZ.edf'
% May 2006, Greg Johnson
%
%> Usage:
% edf_crop(filename_template,filename_range,xrange,yrange)
%> Uses the standard C conventions for the filename_template (file_%04d.edf
% will find all the images with a four digit number at the end).  Using the
% .edf extension is OPTIONAL - works either way
%>filename_range is a vector of file number ([1:1500] or if want
% [0:5:1500])
%>xrange and yrange are also vectors that can be simple ([400:600]) or more
% complex for skipping pixels ([400:10:600]).
%
% By default, EDF_CROP will not replace an existing file.  If you are sure
% you want to, use 'replace' as an additional argument at the end
%
% Examples:
% edf_crop('yourfiles_%05',0:5:1500,1:10,50:75)
% edf_crop('myfiles_%04d.edf',0:1502,100:900,200:5:500,'replace')
% edf_crop('dark',0,100:900,200:5:500);  % (works on single image, too)
% 
% See also:
% EDF_DOWNSAMPLE

% Not much error checking done for correct arguments :(

if nargin==5
  if strcmpi(varargin{1},'replace')
    replace=true;
  else
    replace=false;
  end
else
  replace=false;
end



[fnameroot,fnameformatstring]=sfStripExtensionandFormat(fnametemplate);
fnameintemplate=[fnameroot fnameformatstring '.edf'];
fnameouttemplate=[fnameroot 'crop_' fnameformatstring '.edf'];

if replace
  disp('Replacing any files which already exist!!')
else
  disp('Not replacing existing files')
end
for n=fnamerange
  fnamein=sprintf(fnameintemplate,n);
  fnameout=sprintf(fnameouttemplate,n);
  fprintf('Slice: %d/%d...',n,fnamerange(end));
  if replace==false
    if exist(fnameout,'file')
      fprintf('skipping\n');
    else
      sfReadandCrop
    end
  else
    sfReadandCrop
  end
  fprintf('\r');
end
fprintf('\n')

function sfReadandCrop
    [im,datatype]=edf_read(fnamein); % use edfread2 to get datatype for the edfwrite later...
% image is NOT transposed, so yrange,xrange (rather than vice-versa) IS correct
    im_crop=im(yrange,xrange);
    edf_write(im_crop,fnameout,datatype)

end
end



function [fnameroot,fnameformatstring]=sfStripExtensionandFormat(full_fname)
% removes, if necessary, the .edf extension of a file name
  if strcmpi(full_fname(end-3:end),'.edf')
      fname=full_fname(1:end-4);
  else
    fname=full_fname;
  end
  [fnameroot,fnameformatstring]=sfStripFormatting(fname);
end
  

function [root,fmt]=sfStripFormatting(fname)
% removes, if it can, any formatting at the end of the fname (%04d, for example)

ndx=find(fname=='%');
if ndx
  root=fname(1:ndx-1);
  fmt=fname(ndx:end);
else
  root=fname;
  fmt='';
end
end
