function volume=volread_single(filename,format,skip,xsize,ysize,zsize,byteorder)
% reads a volume in raw format into the matrix image
%
% name of the file (a string)
%
% other parameters are guessed if only the filename is given
%
% format: datatype, see fread for possibilities
% skip: number of bytes skipped at start file
% xsize: first dimension (rows in Matlab)
% ysize: second dimension (columns in Matlab)
% zsize: third dimension (default is 1)
% byteorder: 'b' or 'l' (default); big endian or little endian
% result is double
% origin: wolfgang and peter
%
% modified: greg 3/2006 to output singles instead of doubles (requires half
%                       the memory, useful for large volumes).



defaultformat = 'float32';
defaultzsize = 1;

computertype=computer;
if findstr(computertype,'LNX')
  defaultbyteorder='l';
elseif findstr(computertype,'SOL')
  defaultbyteorder='b';
else
  defaultbyteorder='b';
end

possibleskip=[0 210];

guess=0;
switch nargin
  case 0
    error('Please supply some parameters')
  case 1
    guess = 1;
    byteorder=defaultbyteorder;
  case 4
    ysize = xsize;
    zsize = defaultzsize;
    byteorder=defaultbyteorder;
  case 5
    zsize = defaultzsize;
    byteorder=defaultbyteorder;
  case 6
    byteorder=defaultbyteorder;
end


fid=fopen(filename,'r',byteorder);

if fid == -1
  error(sprintf('File %s does not exist!',filename))
end

if guess
  switch defaultformat
    case 'float32'
      nbytes=4;
  end
  vfile=dir(filename);
  sfile=vfile.bytes;
  possize=sqrt((sfile-possibleskip)/nbytes);
  [a,b]=min(abs(possize-round(possize)));
  if (a==0)
    format=defaultformat;
    skip=possibleskip(b);
    xsize=possize(b);
    ysize=xsize;
    zsize=1;
    disp(sprintf('Size seems %d by %d',xsize,ysize))
  else
    zsize=0;
  end
end % guess


if zsize
  fseek(fid,skip,-1);
  % as we never read doubles (64 bit floats) we can save 50% of the
  % memory by converting to singles (32 bit floats)
  if ~any(find(format=='>'))  % don't muck with the format if the user has already done so!
    format=[format '=>float32'];
  end
  volume=fread(fid,xsize*ysize*zsize,format);
  volume=reshape(volume,[xsize,ysize,zsize]);
else
  disp('Problems estimating sizes, enter them manually.')
  help volread
end
fclose(fid);

