function volviewer(volname,varargin)
% VOLVIEWER.M To be documented shortly...
% Greg Johnson March 2006

app=[];
sfFindFiles;
sfInitialiseVariables;
sfMapVolumes;
sfInitialiseFigure;
%if nargin>0
%  % parse_pv_pairs will not remove any part of the structure...
%  app=parse_pv_pairs(app,varargin);
%end
sfUpdateFigure

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EVERYTHING BELOW THIS LINE IS A SUBFUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function sfFindFiles
    % remove any extension from volname
    [fpath,fname,fext,tmp]=fileparts(volname);
    d=dir([fpath fname '*.vol']);
    app.dim(3)=0;
    disp('Only reading ONE file')
    for n=1:length(d);
      app.file{n}.fname=[fpath d(n).name];
      app.file{n}.dim=sfGetDimensions(app.file{n}.fname);
      app.dim(3)=app.dim(3)+app.file{n}.dim(3);
    end
    app.dim(1:2)=[app.file{1}.dim(1:2)];
    fprintf('Found %d subvolumes\n',n);
    % for the moment, use the .info files - should be replaced with xml
    % later
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  function sfInitialiseVariables
    app.currentOrientation=3;
    app.currentslice=10;
    app.downsample=5;
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function sfMapVolumes
    fprintf('Mapping files...\n')
    % can only do 2GB chunks/map
    app.mmap=[];
    app.chunk=[];
   
    numbytespervoxel=4;
    numbytesperslice=app.file{1}.dim(1)*app.file{1}.dim(2)*numbytespervoxel;
    numslicesperchunk=floor(2^31/numbytesperslice);

    for n=1:length(app.file) % number of subvolume files
      fprintf('File %d\n',n);
      % calculate how many chunks are needed for this file
      app.file{n}.numchunks=ceil(app.file{n}.dim(3)./numslicesperchunk);
      for m=1:app.file{n}.numchunks
        slices=min(app.file{n}.dim(3)-(m-1)*numslicesperchunk,numslicesperchunk);
        fprintf('  Chunk %d (%d slices)\n',m,slices);
        app.mmap{end+1}=memmapfile(app.file{n}.fname,...
          'offset',(m-1)*numbytesperslice*numslicesperchunk,...
          'repeat',1,'format',{'single',[app.file{n}.dim(1:2) slices],'vol'});
        if n==1 & m==1
          app.chunk{end+1}.start=1;
        else
          app.chunk{end+1}.start=app.chunk{end}.end+1;
        end
        app.chunk{end}.end=app.chunk{end}.start+slices-1;
      end
    end
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  function sfInitialiseFigure
    warning('off','Images:initSize:adjustingMag');
    [tmp,dname,fname,ext]=fileparts(volname);
    app.clims=[];
    app.h_fig=figure;
    set(app.h_fig,'CloseRequestFcn',@sfQuit);
    app.firsttime=1;
    set(app.h_fig,'keypressfcn',@sfKeyPress);
  end % sfInitialiseFigure

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  function sfUpdateFigure(varargin)
    if nargin==0

      switch app.currentOrientation
        case 1
          app.im=[];
          for n=1:length(app.mmap)
            app.im=cat(1,app.im,squeeze(app.mmap{n}.data.vol(app.currentslice,1:app.downsample:end,1:app.downsample:end))');
          end
        case 2
          app.im=[];
          for n=1:length(app.mmap)
            app.im=cat(1,app.im,squeeze(app.mmap{n}.data.vol(1:app.downsample:end,app.currentslice,1:app.downsample:end))');
          end
        case 3
          for n=1:length(app.mmap)
            if find(app.currentslice==app.chunk{n}.start:app.chunk{n}.end)
              break
            end
          end
          tmp=app.currentslice-app.chunk{n}.start+1;
          app.im=squeeze(...
            app.mmap{n}.data.vol(1:app.downsample:end,1:app.downsample:end,tmp));
      end % switch
      if app.firsttime
        app.clims=[min(app.im(:)) max(app.im(:))];
        app.firsttime=0;
      end
    end
    figure(app.h_fig)
%    imshow(app.im,app.clims);
imagesc(app.im,app.clims)
    colormap(gray)
    axis image
%    imagesc(app.im,app.clims)

    set(gcf,...
      'name',['Slice ' num2str(app.currentslice)],...
      'numbertitle','off',...
      'menubar','none')
    drawnow;
  end % sfUpdateFigure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  sfKeyPress
  function sfKeyPress(varargin)
    % varargin{2} contains the event details
    c=varargin{2};
    
    switch c.Key
      case 'g'
        app.currentslice=inputwdefaultnumeric('Slice number: ',num2str(app.currentslice));
        sfUpdateFigure
      case 'uparrow'
        sfUpdateFigure;
      case 'downarrow'
        sfUpdateFigure;
      case 'x'
        fprintf('Changing to Y-Z orientation\n')
        app.currentOrientation=1;
        sfUpdateFigure;
      case 'y'
        fprintf('Changing to Z-X orientation\n')
        app.currentOrientation=2;
        sfUpdateFigure;
      case 'z'
        fprintf('Changing to X-Y orientation\n')
        app.currentOrientation=3;
        sfUpdateFigure;
      case 'rightarrow'
        if strcmp(c.Modifier,'shift')
          app.currentslice=app.currentslice+10;
        elseif strcmp(c.Modifier,'control')
          disp('moving')
          while 1
            app.currentslice=app.currentslice+10;
            sfUpdateFigure
          end
        else
          app.currentslice=app.currentslice+1;
        end
        if app.currentslice>app.dim(app.currentOrientation)
          app.currentslice=1;
        end
        sfUpdateFigure;
      case 'leftarrow'
        if strcmp(c.Modifier,'shift')
          app.currentslice=app.currentslice-10;
          
        else
          app.currentslice=app.currentslice-1;
        end
        if app.currentslice<1
          app.currentslice=app.dim(app.currentOrientation);
        end
        sfUpdateFigure;
      case 'return'
        if isempty(c.Modifier) % just a plain keystroke
          % write image to workspace 'im' variable
          disp('Variable ''im'' now contains image');
          assignin('base','im',app.im);
        end
      case 'c'
        disp('Resetting colormap')
        app.clims=[min(app.im(:)) max(app.im(:))];
        sfUpdateFigure
      case '1'
        disp('Increasing contrast')
        app.clims=app.clims*0.95;
        sfUpdateFigure('clims');
      case '2'
        disp('Decreasing contrast')
        app.clims=app.clims*1.05;
        sfUpdateFigure('clims');
      case '3'
        disp('Colormap brighter')
        app.clims=app.clims-max(app.clims)*0.05;
        sfUpdateFigure('clims');
      case '4'
        disp('Colormap darker')
        app.clims=app.clims+max(app.clims)*0.05;
        sfUpdateFigure('clims');
      case 'q'
        sfQuit
    end % switch
  end % sfKeyPress



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function sfQuit(varargin)
    set(app.h_fig,'KeyPressFcn',[]);
    clear app
    disp('Done!')
    closereq
  end
  
  
  
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function dim=sfGetDimensions(tmpname)
    tmpname=[tmpname '.info'];
    if exist(tmpname,'file')
      fid=fopen(tmpname,'rt');
      fgetl(fid);
      tmp=fgetl(fid);
      dim(1)=sscanf(tmp,'NUM_X = %d');
      tmp=fgetl(fid);
      dim(2)=sscanf(tmp,'NUM_Y = %d');
      tmp=fgetl(fid);
      dim(3)=sscanf(tmp,'NUM_Z = %d');
    else
      fprintf('Could not find dimensions of volume in the .info file\n');
      volname
      dim=[];
    end
  end % sfGetDimensions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  end
